/*
 * Copyright 2012 StageBloc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

#if TARGET_OS_IPHONE
#import "NXOAuth2.h"
#else
#import <OAuth2Client/NXOAuth2.h>
#endif

#import "SBAccount.h"
#import "SBAccount+Private.h"
#import "SBConstants.h"

#import "SBRequest.h"

@interface SBRequest ()
@property (nonatomic, retain) NXOAuth2Request *oauthRequest;
@end

@implementation SBRequest

#pragma mark Class Methods

+ (id)   performMethod:(SBRequestMethod)aMethod
            onResource:(NSURL *)aResource
       usingParameters:(NSDictionary *)someParameters
           withAccount:(SBAccount *)anAccount
sendingProgressHandler:(SCRequestSendingProgressHandler)aProgressHandler
       responseHandler:(SCRequestResponseHandler)aResponseHandler;
{
    NSString *theMethod;
    switch (aMethod) {
        case SBRequestMethodPOST:
            theMethod = @"POST";
            break;
            
        case SBRequestMethodPUT:
            theMethod = @"PUT";
            break;
            
        case SBRequestMethodDELETE:
            theMethod = @"DELETE";
            break;
            
        case SBRequestMethodHEAD:
            theMethod = @"HEAD";
            break;
            
        default:
            theMethod = @"GET";
            break;
    }
        
    //NSAssert1([[aResource scheme] isEqualToString:@"https"], @"Resource '%@' is invalid because the scheme is not 'https'.", aResource);
    
    NXOAuth2Request *request = [[NXOAuth2Request alloc] initWithResource:aResource method:theMethod parameters:someParameters];
    request.account = anAccount.oauthAccount;
    [request performRequestWithSendingProgressHandler:aProgressHandler
                                      responseHandler:aResponseHandler];
    return [request autorelease];
}

+ (void)cancelRequest:(id)request;
{
    if ([request isKindOfClass:[NXOAuth2Request class]] ||
        [request isKindOfClass:[SBRequest class]]) {
        [request cancel];
    }
}


#pragma mark Lifecycle

- (id)init;
{
    NSAssert(NO, @"Please use the designated initializer");
    return nil;
}

- (id)initWithMethod:(SBRequestMethod)aMethod resource:(NSURL *)aResource;
{
    self = [super init];
    if (self) {
        
        NSString *theMethod;
        switch (aMethod) {
            case SBRequestMethodPOST:
                theMethod = @"POST";
                break;
                
            case SBRequestMethodPUT:
                theMethod = @"PUT";
                break;
                
            case SBRequestMethodDELETE:
                theMethod = @"DELETE";
                break;
                
            case SBRequestMethodHEAD:
                theMethod = @"HEAD";
                break;
                
            default:
                theMethod = @"GET";
                break;
        }
        
        oauthRequest = [[NXOAuth2Request alloc] initWithResource:aResource method:theMethod parameters:nil];
    }
    return self;
}


#pragma mark Accessors

@synthesize oauthRequest;

- (SBAccount *)account;
{
    if (self.oauthRequest.account) {
        return [[[SBAccount alloc] initWithOAuthAccount:self.oauthRequest.account] autorelease];
    } else {
        return nil;
    }
}

- (void)setAccount:(SBAccount *)account;
{
    self.oauthRequest.account = account.oauthAccount;
}

- (SBRequestMethod)requestMethod;
{
    NSString *aMethod = self.oauthRequest.requestMethod;
    
    if ([aMethod caseInsensitiveCompare:@"POST"]) {
        return SBRequestMethodPOST;
    } else if ([aMethod caseInsensitiveCompare:@"PUT"]) {
        return SBRequestMethodPUT;
    } else if ([aMethod caseInsensitiveCompare:@"DELETE"]) {
        return SBRequestMethodDELETE;
    } else if ([aMethod caseInsensitiveCompare:@"HEAD"]) {
        return SBRequestMethodHEAD;
    } else {
        NSAssert1([aMethod caseInsensitiveCompare:@"GET"], @"SCRequest only supports 'GET', 'PUT', 'POST' and 'DELETE' as request method. Underlying NXOAuth2Accound uses the request method '%@'.", aMethod);
        return SBRequestMethodGET;
    }
}

- (void)setRequestMethod:(SBRequestMethod)requestMethod;
{
    NSString *theMethod;
    switch (requestMethod) {
        case SBRequestMethodPOST:
            theMethod = @"POST";
            break;
            
        case SBRequestMethodPUT:
            theMethod = @"PUT";
            break;
            
        case SBRequestMethodDELETE:
            theMethod = @"DELETE";
            break;
            
        case SBRequestMethodHEAD:
            theMethod = @"HEAD";
            break;
            
        default:
            theMethod = @"GET";
            break;
    }
    self.oauthRequest.requestMethod = theMethod;
}

- (NSURL *)resource;
{
    return self.oauthRequest.resource;
}

- (void)setResource:(NSURL *)resource;
{
    self.oauthRequest.resource = resource;
}

- (NSDictionary *)parameters;
{
    return self.oauthRequest.parameters;
}

- (void)setParameters:(NSDictionary *)parameters;
{
    self.oauthRequest.parameters = parameters;
}


#pragma mark Signed NSURLRequest

- (NSURLRequest *)signedURLRequest;
{
    return [self.oauthRequest signedURLRequest];
}


#pragma mark Perform Request

- (void)performRequestWithSendingProgressHandler:(SCRequestSendingProgressHandler)aSendingProgressHandler
                                 responseHandler:(SCRequestResponseHandler)aResponseHandler;
{
    [self.oauthRequest performRequestWithSendingProgressHandler:aSendingProgressHandler
                                                responseHandler:aResponseHandler];
}

- (void)cancel;
{
    [self.oauthRequest cancel];
}

@end
