/*
 * Copyright 2012 StageBloc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

#if TARGET_OS_IPHONE
#import "NXOAuth2.h"
#else
#import <OAuth2Client/NXOAuth2.h>
#endif

#import "SBAccount.h"
#import "SBAccount+Private.h"
#import "SBRequest.h"
#import "SBConstants.h"

#import "SBStageBloc+Private.h"
#import "SBStageBloc.h"


#pragma mark Notifications

NSString * const SBStageBlocAccountsDidChangeNotification = @"SBStageBlocAccountsDidChangeNotification";
NSString * const SBStageBlocAccountDidChangeNotification = @"SBStageBlocAccountDidChangeNotification";
NSString * const SBStageBlocDidFailToRequestAccessNotification = @"SBStageBlocDidFailToRequestAccessNotification";

#pragma mark -


@interface SBStageBloc ()

#pragma mark Notification Observer
- (void)accountStoreAccountsDidChange:(NSNotification *)aNotification;
- (void)accountStoreDidFailToRequestAccess:(NSNotification *)aNotification;
- (void)accountDidFailToGetAccessToken:(NSNotification *)aNotification;
@end

@implementation SBStageBloc

+ (void)initialize;
{
    [SBStageBloc shared];
}

- (id)init;
{
    self = [super init];
    if (self) {
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(accountStoreAccountsDidChange:)
                                                     name:NXOAuth2AccountStoreAccountsDidChangeNotification
                                                   object:nil];
        
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(accountStoreDidFailToRequestAccess:)
                                                     name:NXOAuth2AccountStoreDidFailToRequestAccessNotification
                                                   object:nil];
        
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(accountDidFailToGetAccessToken:)
                                                     name:NXOAuth2AccountDidFailToGetAccessTokenNotification
                                                   object:nil];
    }
    return self;
}

- (void)dealloc;
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}

#pragma mark Accessors

+ (SBAccount *)account;
{    
    NSArray *oauthAccounts = [[NXOAuth2AccountStore sharedStore] accountsWithAccountType:kSBAccountType];
    if ([oauthAccounts count] > 0) {
        return [[[SBAccount alloc] initWithOAuthAccount:[oauthAccounts objectAtIndex:0]] autorelease];
    } else {
        return nil;
    }
}


#pragma mark Manage Accounts

+ (void)requestAccessWithPreparedAuthorizationURLHandler:(SBPreparedAuthorizationURLHandler)aPreparedAuthorizationURLHandler;
{
    [self removeAccess];
    [[NXOAuth2AccountStore sharedStore] requestAccessToAccountWithType:kSBAccountType
                                   withPreparedAuthorizationURLHandler:(SBPreparedAuthorizationURLHandler)aPreparedAuthorizationURLHandler];
}

+ (void)removeAccess;
{    
    NSArray *accounts = [[NXOAuth2AccountStore sharedStore] accountsWithAccountType:kSBAccountType];
    for (NXOAuth2Account *account in accounts) {
        [[NXOAuth2AccountStore sharedStore] removeAccount:account];
    }
}


#pragma mark Configuration

+ (void)setClientID:(NSString *)aClientID
             secret:(NSString *)aSecret
        redirectURL:(NSURL *)aRedirectURL;
{
    [SBStageBloc setClientID:aClientID secret:aSecret redirectURL:aRedirectURL developmentMode:NO];
}

+ (void)setClientID:(NSString *)aClientID
             secret:(NSString *)aSecret
        redirectURL:(NSURL *)aRedirectURL
    developmentMode:(BOOL)developmentMode;
{
    NSMutableDictionary *config = [NSMutableDictionary dictionary];
    
    [config setObject:aClientID forKey:kNXOAuth2AccountStoreConfigurationClientID];
    [config setObject:aSecret forKey:kNXOAuth2AccountStoreConfigurationSecret];
    [config setObject:aRedirectURL forKey:kNXOAuth2AccountStoreConfigurationRedirectURL];
    
    if (developmentMode) {
        [config setObject:[NSURL URLWithString:kSBStageBlocSandboxAuthURL] forKey:kNXOAuth2AccountStoreConfigurationAuthorizeURL];
        [config setObject:[NSURL URLWithString:kSBStageBlocSandboxAccessTokenURL] forKey:kNXOAuth2AccountStoreConfigurationTokenURL];
        [config setObject:[NSURL URLWithString:kSBStageBlocSandboxAPIURL] forKey:kSBConfigurationAPIURL];
    } else {
        [config setObject:[NSURL URLWithString:kSBStageBlocAuthURL] forKey:kNXOAuth2AccountStoreConfigurationAuthorizeURL];
        [config setObject:[NSURL URLWithString:kSBStageBlocAccessTokenURL] forKey:kNXOAuth2AccountStoreConfigurationTokenURL];
        [config setObject:[NSURL URLWithString:kSBStageBlocAPIURL] forKey:kSBConfigurationAPIURL];        
    }
    
    [[NXOAuth2AccountStore sharedStore] setConfiguration:config forAccountType:kSBAccountType];
}


#pragma mark OAuth2 Flow

+ (BOOL)handleRedirectURL:(NSURL *)URL;
{
    return [[NXOAuth2AccountStore sharedStore] handleRedirectURL:URL];
}

#pragma mark Notification Observer

- (void)accountStoreAccountsDidChange:(NSNotification *)aNotification;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:SBStageBlocAccountDidChangeNotification
                                                        object:self];
}

- (void)accountStoreDidFailToRequestAccess:(NSNotification *)aNotification;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:SBStageBlocDidFailToRequestAccessNotification
                                                        object:self
                                                      userInfo:aNotification.userInfo];
}

- (void)accountDidFailToGetAccessToken:(NSNotification *)aNotification;
{
    [[NSNotificationCenter defaultCenter] postNotificationName:SCAccountDidFailToGetAccessToken
                                                        object:aNotification.object];
}

@end
