/*
 * Copyright 2012 StageBloc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

#import "UIViewController+StageBlocUI.h"
#import "UIDevice+StageBlocUI.h"

#import "SBLoginView.h"
#import "SCConnectToStageBlocTitleView.h"
#import "NXOAuth2AccountStore.h"

#import "SBStageBloc.h"
#import "SBStageBloc+Private.h"
#import "SBConstants.h"
#import "SBBundle.h"

#import "SBUIErrors.h"

#import "SBLoginViewController.h"


#pragma mark -

@interface SBLoginViewController ()

#pragma mark Accessors
@property (nonatomic, retain) NSURL *preparedURL;
@property (nonatomic, assign) SBLoginView *loginView;
@property (nonatomic, copy) SBLoginViewControllerCompletionHandler completionHandler;

#pragma mark Notifications
- (void)accountDidChange:(NSNotification *)aNotification;
- (void)failToRequestAccess:(NSNotification *)aNotification;

#pragma mark Action
- (void)cancel;
@end


@implementation SBLoginViewController


#pragma mark Class Methods

+ (UINavigationController *)loginViewControllerWithPreparedURL:(NSURL *)anURL completionHandler:(SBLoginViewControllerCompletionHandler)aCompletionHandler;
{
    
    SBLoginViewController *loginViewController = [[[SBLoginViewController alloc] initWithPreparedURL:anURL completionHandler:aCompletionHandler] autorelease];
    UINavigationController *navigationController = [[UINavigationController alloc] initWithRootViewController:loginViewController];
    [navigationController setModalPresentationStyle:UIModalPresentationFormSheet];
    
    return [navigationController autorelease];
}

#pragma mark Lifecycle

@synthesize preparedURL;
@synthesize loginView;
@synthesize completionHandler;

- (id)initWithPreparedURL:(NSURL *)anURL completionHandler:(SBLoginViewControllerCompletionHandler)aCompletionHandler;
{
    self = [super init];
    if (self) {
        preparedURL = [anURL retain];
        completionHandler = [aCompletionHandler copy];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(accountDidChange:)
                                                     name:SBStageBlocAccountDidChangeNotification
                                                   object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(failToRequestAccess:)
                                                     name:SBStageBlocDidFailToRequestAccessNotification
                                                   object:nil];
    }
    return self;
}

- (void)dealloc;
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [preparedURL release];
    [completionHandler release];
    
    [super dealloc];
}


#pragma mark UIViewController

- (void)viewDidLoad;
{
    [super viewDidLoad];
    
    self.loginView = [[[SBLoginView alloc] initWithFrame:self.view.bounds] autorelease];
    self.loginView.delegate = self;
    [self.loginView loadURL:self.preparedURL];
    [self.view addSubview:self.loginView];
    
    // Navigation Bar
    self.navigationController.navigationBarHidden = YES;
    
    // Toolbar
    self.navigationController.toolbar.barStyle = UIBarStyleBlack;
    self.navigationController.toolbarHidden = NO;
    
    NSMutableArray *toolbarItems = [NSMutableArray arrayWithCapacity:1];
    
    [toolbarItems addObject:[[[UIBarButtonItem alloc] initWithTitle:SBLocalizedString(@"cancel", @"Cancel")
                                                              style:UIBarButtonItemStyleBordered
                                                             target:self
                                                             action:@selector(cancel)] autorelease]];
    [self setToolbarItems:toolbarItems];
}

- (void)viewWillAppear:(BOOL)animated;
{
    [super viewWillAppear:animated];
    [self.view addSubview:[[[SCConnectToStageBlocTitleView alloc] initWithFrame:CGRectMake(0.0, 0.0, CGRectGetWidth(self.view.bounds), 28.0)] autorelease]];
    self.loginView.frame = CGRectMake(0.0, 28.0, CGRectGetWidth(self.view.bounds), CGRectGetHeight(self.view.bounds) - 28.0);
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation;
{
    if ([UIDevice isIPad]) {
        return YES;
    } else {
        return toInterfaceOrientation != UIInterfaceOrientationPortraitUpsideDown;
    }
}

#pragma mark Notifications

- (void)accountDidChange:(NSNotification *)aNotification;
{
    if (self.completionHandler) {
        self.completionHandler(nil);
    } else {
        [[self modalPresentingViewController] dismissModalViewControllerAnimated:YES];
    }
}

- (void)failToRequestAccess:(NSNotification *)aNotification;
{
    if (self.completionHandler) {
        NSError *error = [[aNotification userInfo] objectForKey:NXOAuth2AccountStoreErrorKey];
        self.completionHandler(error);
    } else {
        [[self modalPresentingViewController] dismissModalViewControllerAnimated:YES];
    }
}


#pragma mark Private

- (void)cancel;
{   
    if (self.completionHandler) {
        NSDictionary *userInfo = [NSDictionary dictionaryWithObject:@"Canceled by user." forKey:NSLocalizedDescriptionKey];
        self.completionHandler([NSError errorWithDomain:SBUIErrorDomain code:SBUICanceledErrorCode userInfo:userInfo]);
    } else {
        [[self modalPresentingViewController] dismissModalViewControllerAnimated:YES];
    }
}


@end
